@@ -1,6 +1,6 @@
 /*
     This file is part of darktable,
-    Copyright (C) 2021-2023 darktable developers.
+    Copyright (C) 2021-2024 darktable developers.
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -125,11 +125,7 @@ void commit_params(dt_iop_module_t *self, dt_iop_params_t *p1, dt_dev_pixelpipe_
 inline static void blur_2D_Bspline(const float *const restrict in, float *const restrict out,
                                    const size_t width, const size_t height)
 {
-#ifdef _OPENMP
-#pragma omp parallel for default(none) dt_omp_firstprivate(width, height, in, out) \
-    schedule(simd: static)    \
-    collapse(2)
-#endif
+  DT_OMP_FOR(collapse(2))
   for(size_t i = 0; i < height; i++)
   {
     for(size_t j = 0; j < width; j++)
@@ -159,10 +155,7 @@ inline static void blur_2D_Bspline(const float *const restrict in, float *const
 static inline void init_kernel(float *const restrict buffer, const size_t width, const size_t height)
 {
   // init an empty kernel with zeros
-#ifdef _OPENMP
-#pragma omp parallel for simd default(none) dt_omp_firstprivate(width, height, buffer) \
-    schedule(simd: static) aligned(buffer:64)
-#endif
+  DT_OMP_FOR_SIMD(aligned(buffer:64))
   for(size_t k = 0; k < height * width; k++) buffer[k] = 0.f;
 }
 
@@ -181,10 +174,7 @@ static inline void create_lens_kernel(float *const restrict buffer,
   const float eps = 1.f / (float)width;
   const float radius = (float)(width - 1) / 2.f - 1;
 
-#ifdef _OPENMP
-#pragma omp parallel for simd default(none) dt_omp_firstprivate(width, height, buffer, n, m, k, rotation, eps, radius) \
-    schedule(simd: static) aligned(buffer:64) collapse(2)
-#endif
+  DT_OMP_FOR(collapse(2))
   for(size_t i = 0; i < height; i++)
     for(size_t j = 0; j < width; j++)
     {
@@ -227,10 +217,7 @@ static inline void create_motion_kernel(float *const restrict buffer,
   const float M[2][2] = { { cosf(corr_angle), -sinf(corr_angle) },
                           { sinf(corr_angle), cosf(corr_angle) } };
 
-#ifdef _OPENMP
-#pragma omp parallel for simd default(none) dt_omp_firstprivate(width, height, buffer, A, B, C, radius, offset, M, eps) \
-    schedule(simd: static) aligned(buffer:64)
-#endif
+  DT_OMP_FOR_SIMD(aligned(buffer:64))
   for(size_t i = 0; i < 8 * width; i++)
   {
     // Note : for better smoothness of the polynomial discretization,
@@ -274,10 +261,7 @@ static inline void create_gauss_kernel(float *const restrict buffer,
   // 2 × 1D convolutions.
   const float radius = (width - 1) / 2.f - 1;
 
-#ifdef _OPENMP
-#pragma omp parallel for simd default(none) dt_omp_firstprivate(width, height, buffer, radius) \
-    schedule(simd: static) aligned(buffer:64) collapse(2)
-#endif
+  DT_OMP_FOR(collapse(2))
   for(size_t i = 0; i < height; i++)
     for(size_t j = 0; j < width; j++)
     {
@@ -325,10 +309,7 @@ static inline void build_gui_kernel(unsigned char *const buffer, const size_t wi
   }
 
   // Convert to Gtk/Cairo RGBA 8×4 bits
-#ifdef _OPENMP
-#pragma omp parallel for simd default(none) dt_omp_firstprivate(width, height, buffer, kernel_2) \
-    schedule(simd: static) aligned(buffer, kernel_2:64)
-#endif
+  DT_OMP_FOR_SIMD(aligned(buffer, kernel_2:64))
   for(size_t k = 0; k < height * width; k++)
   {
     buffer[k * 4] = buffer[k * 4 + 1] = buffer[k * 4 + 2] = buffer[k * 4 + 3] = roundf(255.f * kernel_2[k]);
@@ -343,10 +324,7 @@ static inline float compute_norm(float *const buffer, const size_t width, const
 {
   float norm = 0.f;
 
-#ifdef _OPENMP
-#pragma omp parallel for simd default(none) dt_omp_firstprivate(width, height, buffer) \
-    schedule(simd: static) aligned(buffer:64) reduction(+:norm)
-#endif
+  DT_OMP_FOR_SIMD(aligned(buffer:64) reduction(+:norm))
   for(size_t i = 0; i < width * height; i++)
   {
     norm += buffer[i];
@@ -358,10 +336,7 @@ static inline float compute_norm(float *const buffer, const size_t width, const
 
 static inline void normalize(float *const buffer, const size_t width, const size_t height, const float norm)
 {
-#ifdef _OPENMP
-#pragma omp parallel for simd default(none) dt_omp_firstprivate(width, height, buffer, norm) \
-    schedule(simd: static) aligned(buffer:64)
-#endif
+  DT_OMP_FOR_SIMD(aligned(buffer:64))
   for(size_t i = 0; i < width * height; i++)
   {
     buffer[i] /= norm;
@@ -441,10 +416,7 @@ static void process_fft(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *pi
   }
 
   // Write the image in the padded buffer
-#ifdef _OPENMP
-#pragma omp parallel for simd default(none) dt_omp_firstprivate(padded_width, padded_height, roi_in, in, padded_in) \
-    schedule(simd: static) aligned(in, padded_in:64)
-#endif
+  DT_OMP_FOR_SIMD(aligned(in, padded_in:64))
   for(size_t i = 0; i < roi_in->height; i++)
     for(size_t j = 0; j < roi_in->width; j++)
     {
@@ -456,10 +428,7 @@ static void process_fft(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *pi
   // Write the padding if needed
   if(padded_width > roi_in->width)
   {
-#ifdef _OPENMP
-#pragma omp parallel for simd default(none) dt_omp_firstprivate(padded_width, padded_height, roi_in, in, padded_in) \
-    schedule(simd: static) aligned(in, padded_in:64)
-#endif
+  DT_OMP_FOR_SIMD(aligned(in, padded_in:64))
   for(size_t i = 0; i < roi_in->height; i++)
     {
       const size_t index_in = (i * (roi_in->width - 1)) * 4;
@@ -470,10 +439,7 @@ static void process_fft(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *pi
 
   if(padded_height > roi_in->height)
   {
-#ifdef _OPENMP
-#pragma omp parallel for simd default(none) dt_omp_firstprivate(padded_width, padded_height, roi_in, in, padded_in) \
-    schedule(simd: static) aligned(in, padded_in:64)
-#endif
+  DT_OMP_FOR_SIMD(aligned(in, padded_in:64))
   for(size_t j = 0; j < roi_in->width; j++)
     {
       const size_t index_in = ((roi_in->height - 1) * roi_in->width + j) * 4;
@@ -496,11 +462,7 @@ static void process_fft(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *pi
   const size_t i_reach = offset_i + kernel_width;
   const size_t j_reach = offset_j + kernel_width;
 
-#ifdef _OPENMP
-#pragma omp parallel for simd default(none) \
-    dt_omp_firstprivate(padded_width, padded_height, padded_kernel, kernel, offset_i, offset_j, i_reach, j_reach, kernel_width) \
-    schedule(simd: static) aligned(kernel, padded_kernel:64)
-#endif
+  DT_OMP_FOR_SIMD(aligned(kernel, padded_kernel:64))
   for(size_t i = 0; i < padded_width; i++)
     for(size_t j = 0; j < padded_width; j++)
     {
@@ -574,8 +536,8 @@ void process(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece,
   if(!dt_iop_have_required_input_format(4, self, piece->colors, ivoid, ovoid, roi_in, roi_out))
     return;
 
-  const float *const restrict in = __builtin_assume_aligned(ivoid, 64);
-  float *const restrict out = __builtin_assume_aligned(ovoid, 64);
+  const float *const restrict in = DT_IS_ALIGNED(ivoid);
+  float *const restrict out = DT_IS_ALIGNED(ovoid);
 
   // Init the blur kernel
   const int radius = MAX(roundf(p->radius / scale), 2);
@@ -584,11 +546,7 @@ void process(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece,
   float *const restrict kernel = dt_alloc_align_float(kernel_width * kernel_width);
   build_pixel_kernel(kernel, kernel_width, kernel_width, p);
 
-#ifdef _OPENMP
-#pragma omp parallel for default(none) \
-    dt_omp_firstprivate(roi_out, in, out, kernel, kernel_width, radius) \
-    schedule(simd: static) collapse(2)
-#endif
+  DT_OMP_FOR(collapse(2))
   for(int i = 0; i < roi_out->height; i++)
     for(int j = 0; j < roi_out->width; j++)
     {
@@ -765,7 +723,7 @@ static gboolean dt_iop_tonecurve_draw(GtkWidget *widget, cairo_t *crf, gpointer
 
   if(!g->img_cached)
   {
-    g->img = dt_alloc_align(64, sizeof(unsigned char) * 4 * allocation.width * allocation.width);
+    g->img = dt_alloc_align_type(unsigned char, 4 * allocation.width * allocation.width);
     g->img_width = allocation.width;
     g->img_cached = TRUE;
     build_gui_kernel(g->img, g->img_width, g->img_width, p);
@@ -807,7 +765,7 @@ void gui_init(dt_iop_module_t *self)
   g->img = NULL;
   g->img_width = 0.f;
 
-  g->area = GTK_DRAWING_AREA(dtgtk_drawing_area_new_with_aspect_ratio(1.f));
+  g->area = GTK_DRAWING_AREA(dtgtk_drawing_area_new_with_height(0));
   g_signal_connect(G_OBJECT(g->area), "draw", G_CALLBACK(dt_iop_tonecurve_draw), self);
   gtk_box_pack_start(GTK_BOX(self->widget), GTK_WIDGET(g->area), TRUE, TRUE, 0);
 
